@@ -28,8 +28,11 @@ DIVISIONS = """
 MONTHS = dict(
     (m.upper(), i) for i, m in enumerate(calendar.month_abbr) if m
 )
-MULTI_KEYWORDS = set(["reference"])
 
+MULTI_KEYWORDS = set(["reference"])
+MULTI_QUALIFIERS = set("""
+    allele db_xref ec_number go_component go_process go_function transl_except
+""".split())
 #
 # Parse Actions
 #
@@ -64,6 +67,10 @@ norm_inlinewhite = OneOrMore(" \t").setParseAction(replaceWith(" "))
 sup_hyphen = Suppress("-")
 sup_colon = Suppress(":")
 sup_semicolon = Suppress(";")
+sup_comma = Suppress(",")
+sup_lparen = Suppress("(")
+sup_rparen = Suppress(")")
+sup_space = Suppress(" ")
 matchall = Regex("(.|\n)*")
 
 accession = Word(alphas, bodyChars=alphanums + "_")
@@ -94,7 +101,7 @@ keyword = Combine(
 )
 keywordsection = OneOrMore(keyword)
 
-optspace = Optional(" ").suppress()
+optspace = Optional(sup_space)
 keywordfirst = ZeroOrMore(Word(printables) + optspace) + LineEnd().suppress()
 keywordcont = White(exact=12).suppress() + keywordfirst
 keywordnorm = Combine(keywordfirst + ZeroOrMore(keywordcont), joinString=" ")
@@ -105,7 +112,7 @@ definition = Literal("DEFINITION").suppress() + normalized
 # ACCESSION
 accfirst = accession.setResultsName("first")
 acclast = accession.setResultsName("last")
-accrange = Group(accfirst + Suppress("-") + acclast)
+accrange = Group(accfirst + sup_hyphen + acclast)
 accession = Literal("ACCESSION").suppress() + OneOrMore(accrange ^ accession)
 
 # VERSION
@@ -126,6 +133,9 @@ dbln_type = dbln_type.setParseAction(lambda t: t[0].lower())
 dbln_list = Group(delimitedList(Word(alphanums + "_")))("identifiers")
 dblink = Group(dbln_type + sup_colon + dbln_list)("dblink")
 
+# PROJECT
+project = Literal("PROJECT").suppress() + normalized
+
 # KEYWORDS
 kwphrase = ZeroOrMore(Word(''.join(p for p in printables if p != ";")))
 kwphrase = Combine(kwphrase, joinString=" ", adjacent=False)
@@ -141,7 +151,7 @@ segment = Literal("SEGMENT").suppress() \
 sourcename = Literal("SOURCE").suppress() + keywordnorm
 organism = Literal("ORGANISM").suppress() + Combine(WordStart() + restOfLine)
 taxname = Combine(OneOrMore(
-    Word(filter(lambda p: p != ";", printables)) + Suppress(Optional(" "))
+    Word(filter(lambda p: p != ";", printables)) + Optional(sup_space)
 ), joinString=" ", adjacent=True)
 taxonomy = delimitedList(OneOrMore(taxname), delim=";")
 source = sourcename + organism + taxonomy
@@ -150,7 +160,7 @@ source = sourcename + organism + taxonomy
 basesorresidues = Suppress("bases") | Suppress("residues")
 position = basesorresidues + integer("from") + Suppress("to") + integer("to")
 notes = Word(''.join(p for p in printables if p not in "()"))("notes")
-refloc = Suppress("(") + Optional(position) + Optional(notes) + Suppress(")")
+refloc = sup_lparen + Optional(position) + Optional(notes) + sup_rparen
 refdesc = integer("refid") + Group(refloc)
 authors = Group(Literal("AUTHORS") + keywordnorm)
 title = Group(Literal("TITLE") + keywordnorm)
@@ -164,7 +174,7 @@ reference = Literal("REFERENCE").suppress() + refdesc + refsubs
 # PRIMARY
 priheaderliterals = "REFSEQ_SPAN PRIMARY_IDENTIFIER PRIMARY_SPAN COMP".split()
 priheader = And(map(Literal, priheaderliterals)).suppress()
-pri_span = Group(integer("from") + Suppress("-") + integer("to"))
+pri_span = Group(integer("from") + sup_hyphen + integer("to"))
 pri_row = Group(pri_span + veracc + pri_span + Optional("c"))
 primary = Literal("PRIMARY").suppress() + priheader + OneOrMore(pri_row)
 
@@ -173,20 +183,20 @@ comment = Literal("COMMENT").suppress() + dedented
 
 # LOCATION
 location = Forward()
-args = (Suppress("(") + delimitedList(location) + Suppress(")"))
+args = (sup_lparen + delimitedList(location) + sup_rparen)
 index = Group(Optional(Word("<>", exact=1)) + integer)
-intargs = (Suppress("(") + delimitedList(integer) + Suppress(")"))
+intargs = (sup_lparen + delimitedList(integer) + sup_rparen)
 bond = Group(Literal("bond") + intargs)("bond")
 one_of = Group(Literal("one-of") + intargs)("one-of")
 position = index | one_of
 span = Group(position + Suppress("..") + position)
 site = Group(index + Suppress("^") + index)
 choice = Group(index + Suppress(".") + index)
-gap = Group(Literal("gap") + Suppress("(") + integer + Suppress(")"))
+gap = Group(Literal("gap") + sup_lparen + integer + sup_rparen)
 complement = Group(Literal("complement") + args)
 join = Group(Literal("join") + args)("join")
 order = Group(Literal("order") + args)("order")
-referenceloc = Group(veracc + Suppress(":") + location)("reference")
+referenceloc = Group(veracc + sup_colon + location)("reference")
 location << (
     complement | join | order | bond | gap | referenceloc \
         | span | site | choice | index | one_of
@@ -221,6 +231,12 @@ contig = Literal("CONTIG") + location
 single_count = Group(integer + Word(alphas))
 base_count = Literal("BASE COUNT") + OneOrMore(single_count)
 
+# TRANSL_EXCEPT
+#(pos:11930596..11930598,aa:Asp)
+mod_pos = Literal("pos").suppress() + sup_colon + location
+mod_acid = Literal("aa").suppress() + sup_colon + Word(alphas)
+modification = sup_lparen + mod_pos + sup_comma + mod_acid + sup_rparen
+
 
 #
 # Keyword Parsers
@@ -253,6 +269,9 @@ def _k_segment(data):
     res = segment.parseString(data)
     return {"position": res.position, "of": res.total}
 
+def _k_project(data):
+    return project.parseString(data)[0]
+
 def _k_keywords(data):
     res = keywords.parseString(data)
     return [kw.rstrip(".") for kw in res]
@@ -397,8 +416,13 @@ for l in loctypenames:
 # Qualifier Parsers
 #
 
+def _q_transl_except(d):
+    res = modification.parseString(d)
+    return (res[0], res[1])
+
 _qual_default = lambda d: d.strip('"')
 _q_allele = _qual_default
+_q_anticodon = _q_transl_except
 _q_bond_type = _qual_default
 _q_chromosome = _qual_default
 _q_clone = _qual_default
@@ -411,6 +435,8 @@ _q_db_xref = _qual_default
 _q_dev_stage = _qual_default
 _q_ec_number = _qual_default
 _q_evidence = _qual_default
+_q_exception = _qual_default
+_q_function = lambda d: " ".join(d.split())
 _q_gene = _qual_default
 _q_go_component = _qual_default
 _q_go_function = _qual_default
@@ -419,6 +445,7 @@ _q_isolate = _qual_default
 _q_map = _qual_default
 _q_mol_type = _qual_default
 _q_note = lambda d: " ".join(d.split())
+_q_ncrna_class = _qual_default
 _q_number = lambda d: int(d)
 _q_organelle = _qual_default
 _q_organism = _qual_default
@@ -426,7 +453,9 @@ _q_product = _qual_default
 _q_protein_id = _qual_default
 _q_pseudo = lambda d: True
 _q_region_name = _qual_default
+_q_ribosomal_slippage = lambda d: True
 _q_rpt_family = _qual_default
+_q_selenocysteine = lambda d: True
 _q_site_type = _qual_default
 _q_standard_name = _qual_default
 _q_strain = _qual_default
@@ -436,7 +465,7 @@ _q_transl_table = _qual_default
 _q_translation = lambda d: ''.join(d.split())
 _q_virion = lambda d: True
 
-MULTI_QUALIFIERS = set(["allele", "db_xref"])
+
 def _add_qualifier(s, quals, name, data):
     if name not in MULTI_QUALIFIERS:
         if name in quals:
